% Create a cluster of hexagons, given the center point coordinates, ISD (
% Inter-Site Distance), the number of layers of hexagons to be created,
% and the index of the eNB in the center
function eNB = CreateCell(eNB,X_center,Y_center,Layers,eNBIx,NeighborCellDistance,NumeNBPerCluster)

eNB(eNBIx).X = X_center; 
eNB(eNBIx).Y = Y_center;
CentereNBIx = eNBIx;

% Start from the center eNB (eNB(1)), find out the other eNBs' coordinates 
% layer by layer. The second layer has 6 eNBs, third has 12, forth has 18, ...
for layer = 2:Layers
    points = 6 * (layer-1); % Number of eNBs in that layer
    for point = 1:points
        % For each layer, Find the haxagon center point located at 30 degree 
        % measured counter-clockwisely from east first, then find other 
        % positions counter-clockwisely
        degree = 360/points*(point-1) + 30; 
        if degree == 30 || degree == 90 || degree == 150 || degree == 210 || degree == 270 ||degree == 330 
            % If the eNB is at 30, 90, 150, 210, 270, 330 degree direction, measured from east.
            % These haxagons have the properties that the distance from its center to the center
            % of the whole graph are equal to NeighborCellDistance * (layer-1)
            eNBIx = eNBIx + 1;
            eNB(eNBIx).X = eNB(CentereNBIx).X + NeighborCellDistance * (layer-1) * cos(degree/360*2*pi);
            eNB(eNBIx).Y = eNB(CentereNBIx).Y + NeighborCellDistance * (layer-1) * sin(degree/360*2*pi);
            tempX = eNB(eNBIx).X;
            tempY = eNB(eNBIx).Y;
            tempDegree = degree;
            count = 1;
        else
            eNBIx = eNBIx + 1;
            eNB(eNBIx).X = tempX + NeighborCellDistance * count * cos((tempDegree+120)/360*2*pi);
            eNB(eNBIx).Y = tempY + NeighborCellDistance * count * sin((tempDegree+120)/360*2*pi);
            count = count + 1;
        end
    end
end
% Find the boundaries of the haxagons
eNBIx = eNBIx - NumeNBPerCluster;
for i = 1:NumeNBPerCluster
    for j = 1:6
        eNB(eNBIx+i).Boundary(j,1) = eNB(eNBIx+i).X + NeighborCellDistance/sqrt(3) * cos(2*pi*(j-1)/6);
        eNB(eNBIx+i).Boundary(j,2) = eNB(eNBIx+i).Y + NeighborCellDistance/sqrt(3) * sin(2*pi*(j-1)/6);
    end
end