function [x] = PerPRB_Power_Optimization(rho,r1,rho1,rho2,rho3,r2,method)

options = optimset('Display', 'off') ;
NumTerms = length(rho1);

if strcmp(method,'optimal')
    % Use fmincon to solve the FeICIC problem directly
    string = ['@(x)-log(log(1+x*',num2str(rho),')+',num2str(log(2)*r1),')'];
    for i = 1:NumTerms
        string = [string '-log(log(1+',num2str(rho1(i)),'/(x*',num2str(rho2(i)),'+',num2str(rho3(i)),'))+',num2str(log(2)*r2(i)),')'];
    end
    fun = str2func(string);
    x = fmincon(fun,1,[1;-1],[1;0],[],[],[],[],[],options);
elseif strcmp(method,'ccp') || strcmp(method,'CCP')
    % Use the concave-convex procedure (CCP) to solve the FeICIC problem
    x = 10^-3; % Initialize the power allocation factor to some value close to zero
    x_temp = -1;
    difference = 10^-2; % Criterion of stopping the CCP, i.e., if the difference between two iterations is smaller than this value
    iteration = 0;
    while abs(x-x_temp) > difference && iteration < 500
        iteration = iteration + 1;
        x_temp = x;
        g_string = ''; % The function g as a string
        for i = 1:NumTerms
            g_string = [g_string '+log( log( 1+',num2str(rho1(i)),'/(',num2str(rho2(i)),'*x+',num2str(rho3(i)),') )+',num2str(log(2)*r2(i)),' )'];
        end
        g_string(1) = ''; % Remove the first + sign
        g = str2func(['@(x)' g_string]);
        
        g_deri_string =''; % The derivative of g as a string
        for i = 1:NumTerms
            g_deri_string = [g_deri_string '-',num2str(rho1(i)*rho2(i)),'/( (',num2str(rho2(i)),'*x+',num2str(rho3(i)),')*(',num2str(rho2(i)),'*x+',num2str(rho3(i)+rho1(i)),')*(log(1+',num2str(rho1(i)),'/(',num2str(rho2(i)),'*x+',num2str(rho3(i)),'))+log(2)*',num2str(r2(i)),') )'];
        end
        g_deri = str2func(['@(x)' g_deri_string]);
        
        g_hat_string = '';
        if feval(g_deri,x_temp) > 0 % But it seems this case will never happen
            g_hat_string = [num2str(feval(g,x_temp)),'+',num2str(feval(g_deri,x_temp)),'*(x-',num2str(x_temp),')'];
        elseif feval(g_deri,x_temp) < 0
            g_hat_string = [num2str(feval(g,x_temp)),'',num2str(feval(g_deri,x_temp)),'*(x-',num2str(x_temp),')'];
        elseif  feval(g_deri,x_temp) == 0
            g_hat_string = [num2str(feval(g,x_temp))];
        end
        
        f_string = ['log(log(1+x*',num2str(rho),')+',num2str(log(2)*r1),')'];
        
        objective_string = ['@(x)-' f_string '-(' g_hat_string ')'];
        
        try
            str2func(objective_string);
        catch ME
            keyboard
        end
        
        try
            x = fmincon(str2func(objective_string),1,[1;-1],[1;0],[],[],[],[],[],options);
        catch ME
            keyboard
        end

        x = fmincon(str2func(objective_string),1,[1;-1],[1;0],[],[],[],[],[],options);
    end
end

clear difference f_string g g_deri g_deri_string g_hat_string g_string i iteration method NumTerms objective_string...
    options r1 r2 rho rho1 rho2 rho3 x_temp