% Calculate throughput for a number of UEs at a time
% The UE must be attached to the station bearing the name StationName
function [UE] = Throughput(StationName,UE,NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots)

if ~isempty(UE)
    % Find out the total interference power assumeing every nearby stations is
    % interfering and everyone of them uses max power
    InfPowerAll = [UE.RSS] .* [UE.ABS_Interfere];
    InfPowerAll_FullPower = [UE.RSS]; % Assume all nearby stations transmit at full power
    % Find out the serving station's power
    ServeIx = find([UE.InterfereMat]==StationName); %- NumInterferer*[0:length(UE)-1]';
    ServePower = InfPowerAll(ServeIx)';
    ServePower_FullPower = InfPowerAll_FullPower(ServeIx)'; % Assume all nearby stations transmit at full power
    % Find the sum of total interference power
    SumInfPower = sum(InfPowerAll, 1);
    SumInfPower_FullPower = sum(InfPowerAll_FullPower, 1); % Assume all nearby stations transmit at full power
    % Find out the SINR per PRB per UE and then the throughput of each UE
    SINR = ServePower ./ (SumInfPower - ServePower + 10^(NoiseDen/10)/1000*BWperRB * 10^(NoiseFigure/10) * ones(1,NumTimeSlots*NumFreqSlots*length(UE)));
    % Assume all nearby stations transmit at full power (WHY do we need this? !!!-!!!)
    SINR_FullPower = ServePower_FullPower ./ (SumInfPower_FullPower - ServePower_FullPower + 10^(NoiseDen/10)/1000*BWperRB * 10^(NoiseFigure/10) * ones(1,NumTimeSlots*NumFreqSlots*length(UE)));
    SINR_FullPower = reshape(SINR_FullPower,NumTimeSlots*NumFreqSlots,length(UE))';
    RatePerPRB = reshape(BWperRB*log2(1+SINR),NumTimeSlots*NumFreqSlots,length(UE))' / BWperRB; % Rounding is applied in the hope of fixing some precision issues in the price-eq algorithm
    NumPRBOccupy = sum([UE.Allocation],2);
    rate = sum(RatePerPRB.*reshape([UE.Allocation],NumTimeSlots*NumFreqSlots,length(UE))',2);
    % Assign values
    for i = 1:length(UE)
        UE(i).ThroughputPerPRB = RatePerPRB(i,:);
        UE(i).throughput = rate(i);
        UE(i).SINR_PerPRB_FullPower = SINR_FullPower(i,:);
    end
end