function [X] = cake_cutting(alpha,U)
% N: no. of agents
% Q:  no. of good
% alpha:  entitlement
% U :  utility 

[N Q] = size(U);

DisplayGraph = 0; % Indicate whether the flow graphs should be plotted

alpha = alpha/sum(alpha); % normalize the vector alpha

if length(alpha)~=N
    error('length of alpha must equal to number of rows in U');
end

if N > 1
    p = max(U);
else
    p = U;
end
p = p/sum(p); % initial price vector


precision = 1e-10;

V = U/diag(p); % ratio of utility vs price
m = max(V');   % maximum utility-price ratio

B=zeros(N,Q);
% find the incidence matrix of option set
for n=1:N
  B(n,:) = (abs(V(n,:) - m(n))<precision);
end
% Create the graph
G = zeros(1+N+Q+1);
G(1,2:1+N) = alpha; % Entitlement of agents
G(2+N:1+N+Q,2+N+Q) = p'; % Price of goods
G(2:1+N,2+N:1+N+Q) = B * 2; 
G_sparse = sparse(G);
% h=view(biograph(G_sparse,[],'ShowWeights','on'));
% set(h.Nodes,'Color',[0 1 0.5])


[F S_plus S_minus J_plus J_minus] = max_flow_FF(alpha,p,B);     % max-flow algorithm
% Display the graph
if DisplayGraph
    Flow = zeros(1+N+Q+1);
    Flow(1,2:1+N) = sum(F,2)';
    Flow(2+N:1+N+Q,2+N+Q) = sum(F,1)';
    Flow(2:1+N,2+N:1+N+Q) = F;
    Flow_sparse = sparse(Flow);
    h=view(biograph(Flow_sparse,[],'ShowWeights','on'));
    set(h.Nodes,'Color',[0 1 0.5])
end

S_0 = ones(1,N) - S_plus  - S_minus;
J_0 = ones(1,Q) - J_plus  - J_minus;
if (min(S_0)<0 | min(J_0)<0)
    error('error in S_0 or J_0')
end

max_excess = S_plus*alpha' - (S_plus*B>0)*p';   % maximal excess value
it = 0;
while (max_excess>precision & max(S_minus)>0 & max(S_plus)>0)   
    % repeat until maximal excess is non-positive
    it = it + 1;
	if((sum(S_plus)>0) & (sum(J_minus>0))) % third inequality in section 3.2
      x = find(S_plus==1);
      y = find(J_minus == 1);
      [M,P]=ndgrid(m(x), p(y));
      epsilon1=min(min( (M.*P-U(x,y))./(U(x,y)*(p*J_minus') + M.*P*(p*J_plus'))  ));
	else
      error('error in S_plus or J_minus'); % should not reach here
	end


    if (sum(S_0)>0)    % if S_0 is non-empty ( a relatively rare event)
        if (sum(J_0)==0)   % sanity check
            error('J_0 is empty when S_0 is not empty');
        end
        x = find(S_plus==1);           % first inequality in section 3.2
        y = find(J_0 == 1);
        [M,P]=ndgrid(m(x), p(y));
        temp = min(min((M.*P./(U(x,y)+1e-20)-1)/(p*J_minus')));
        epsilon1 = min(epsilon1,temp);
		
        x = find(S_0==1);           % second inequality in section 3.2 
        y = find(J_minus == 1);
        [M,P]=ndgrid(m(x), p(y));
        temp = min(min( (1-U(x,y)./(M.*P))  /(p*J_plus')));
        epsilon1 = min(epsilon1,temp);
    end
	        
    epsilon = epsilon1;
    
	if epsilon<=0
        UE_weight = alpha
        ThroughputProfile = U
        error('negative epsilon');
	end
	
    s_plus = S_plus;  % save for future use
    s_minus = S_minus;
    j_plus = J_plus;
    j_minus = J_minus;  
        
    max_excess_in_S_minus = 1;
    while(max_excess_in_S_minus > precision)
       q = p;             % q is p_eta in the paper  
       index = find(J_plus);
       q(index) = q(index) * (1+epsilon* p*J_minus');
       index = find(J_minus);
       q(index) = q(index) * (1-epsilon* p*J_plus');

       alpha1 = alpha;    % set the capacity of some links to zero
       index = find(S_minus==0);  % thus effectively remove every node not in S_minus from the graph
       alpha1(index) = zeros(size(index));
       p1 = q; 
       index = find(J_minus==0); % remove every node not in J_minus
       p1(index) = zeros(size(index));
       
       [F s_plus s_minus j_plus j_minus] = max_flow_FF(alpha1,p1,B);
           
       max_excess_in_S_minus = s_plus*alpha' - (j_plus & J_minus)*p1';
       if (max_excess_in_S_minus> precision)
           epsilon = (1- (alpha*s_plus') / (p*(j_plus & J_minus)'))/ (p*J_plus');
           if (epsilon <=0)
            error('non-positive epsilon')
        end
       end
    end
    
    % update price vector
    q=p;
    index = find(J_plus);
    q(index) = q(index) * (1+epsilon* p*J_minus');
    index = find(J_minus);
    q(index) = q(index) * (1-epsilon* p*J_plus');
    p=q;
    
    V = U/diag(p); % ratio of utility vs price
    m = max(V');   % maximum utility-price ratio
    B= zeros(N,Q);
    % find the incidence matrix of option set
    for n=1:N
        B(n,:) = (abs(V(n,:) - m(n))<precision);
    end
    
    [F S_plus S_minus J_plus J_minus] = max_flow_FF(alpha,p,B);     % max-flow algorithm
    % Display the graph
    if DisplayGraph
        Flow = zeros(1+N+Q+1);
        Flow(1,2:1+N) = sum(F,2)';
        Flow(2+N:1+N+Q,2+N+Q) = sum(F,1)';
        Flow(2:1+N,2+N:1+N+Q) = F;
        Flow_sparse = sparse(Flow);
        h=view(biograph(Flow_sparse,[],'ShowWeights','on'));
        set(h.Nodes,'Color',[0 1 0.5])
    end
    S_0 = ones(1,N) - S_plus  - S_minus;
    J_0 = ones(1,Q) - J_plus  - J_minus;
    if (min(S_0)<0 | min(J_0)<0)
        error('error in S_0 or J_0')
    end
    max_excess = S_plus*alpha' - (S_plus*B>0)*p';
    
    
end

V = U/diag(p); % ratio of utility vs price
m = max(V');   % maximum utility-price ratio

B=zeros(N,Q);
% find the incidence matrix of option set
for n=1:N
    B(n,:) = (abs(V(n,:) - m(n))<precision);
end
X = max_flow_FF(alpha,p,B);
% Display the graph
if DisplayGraph
    Flow = zeros(1+N+Q+1);
    Flow(1,2:1+N) = sum(X,2)';
    Flow(2+N:1+N+Q,2+N+Q) = sum(X,1)';
    Flow(2:1+N,2+N:1+N+Q) = X;
    Flow_sparse = sparse(Flow);
    h=view(biograph(Flow_sparse,[],'ShowWeights','on'));
    set(h.Nodes,'Color',[0 1 0.5])
end
X = X/diag(p);    % normalize each column in X


