% Round-robin scheduler, assign users to resource blocks one-by-one. Time
% slots of the first frequency subcarrier are filled first, then time slots
% of the second frequency subcarrier are filled, etc..
function [station,UE,AttachList] = round_robin_scheduler(station,UE,NumTimeSlots,NumFreqSlots)

% Find all UEs who are attached to the station
AttachList = find([UE.attach]==station.NameTag);
station.UEinPRB = zeros(size(station.UEinPRB,1),size(station.UEinPRB,2));
if ~isempty(AttachList) % If there are users attached to this station
    % Find out the non-zero positions in abs
    available = find(station.ABS);
    % Allocate users to the RBs
    allocation = [repmat(AttachList,1,floor(length(available)/length(AttachList))) AttachList];
    allocation(length(available)+1:length(allocation)) = [];
    station.UEinPRB(available) = allocation;
end

% The following update allocation patterns. 
temp = AttachList'*ones(1,NumTimeSlots*NumFreqSlots)==ones(length(AttachList),1)*station.UEinPRB;
for i = 1:length(AttachList)
    UE(AttachList(i)).Allocation = temp(i,:);
end